/*
 * Frame.h
 *
 *  Created on: March, 2016
 *      Author: anonymous ECCV submission 1747
 */

#ifndef FRAME_H_
#define FRAME_H_

#include <opencv2/core/core.hpp>
#include <opencv2/core/mat.hpp>
#include <opencv2/core/operations.hpp>
#include <deque>
#include <map>
#include <utility>

#ifdef __linux__
#include <cstdbool>
#endif
#include <memory>
#include <string>

#include "Types.h"
#include "IO.h"

namespace nl_uu_science_gmt
{

class Frame
{
	UINT m_id;
	UINT m_index;
	UINT m_timestamp;

	SFramePtr m_self;
	Video* m_video;

	std::string m_image_filename;
	std::string m_flow_filename;

	cv::Size m_flow_size_x;
	cv::Size m_flow_size_y;
	int m_flow_type_x;
	int m_flow_type_y;

	cv::Mat m_frame_image;

	SFramePtr m_next_frame;
	SFramePtr m_this_frame;
	SFramePtr m_prev_frame;

	Skeletons m_skeletons;

	MatPtr m_rotation;
	MatPtr m_translation;

	std::map<SSkelPtr, std::pair<SSkelPtr, cv::Point2d>> m_skel_offsets;

	cv::Point3d m_tlb;
	cv::Point3d m_brf;

	void setup();

public:
	void init(
			const int, const SFramePtr &);

	std::deque<SFramePtr> advance(
			int = 1, int = 0) const;
	std::deque<SFramePtr> regress(
			int = 1, int = 0) const;

	void getImage(
			cv::Mat &) const;
	void getImageCopy(
			cv::Mat &) const;
	void releaseImage();

	void addSkeleton(
			const Skeleton &);

	void write(
			cv::FileStorage &) const;
	void read(
			const cv::FileNode &);

	UINT getId() const
	{
		return m_id;
	}

	void setId(
			UINT id)
	{
		m_id = id;
	}

	UINT getTimeStamp() const
	{
		return m_timestamp;
	}

	void setTimeStamp(
			UINT ts)
	{
		m_timestamp = ts;
	}

	const Video* getVideo() const
	{
		return m_video;
	}

	void setVideo(
			Video* video)
	{
		m_video = video;
	}

	const Skeletons& getSkeletons() const
	{
		return m_skeletons;
	}

	const MatPtr& getRotation() const
	{
		return m_rotation;
	}

	void setRotation(
			const MatPtr& rotation = nullptr)
	{
		m_rotation = rotation;
	}

	const MatPtr& getTranslation() const
	{
		return m_translation;
	}

	void setTranslation(
			const MatPtr& translation = nullptr)
	{
		m_translation = translation;
	}

	const std::string& getImageFilename() const
	{
		return m_image_filename;
	}

	void setImageFilename(
			const std::string& imageFilename)
	{
		m_image_filename = imageFilename;
	}

	const SFramePtr &getNextFrame() const
	{
		return m_next_frame;
	}

	void setNextFrame(
			const SFramePtr &nextFrame)
	{
		m_next_frame = nextFrame;
	}

	const SFramePtr &getPrevFrame() const
	{
		return m_prev_frame;
	}

	void setPrevFrame(
			const SFramePtr &prevFrame)
	{
		m_prev_frame = prevFrame;
	}

	const SFramePtr &getThisFrame() const
	{
		return m_this_frame;
	}

	void setThisFrame(
			const SFramePtr &thisFrame)
	{
		m_this_frame = thisFrame;
	}

	const cv::Mat& getFrameImage() const
	{
		return m_frame_image;
	}

	void setFrameImage(
			const cv::Mat& frameImage)
	{
		m_frame_image = frameImage;
	}

	const std::map<SSkelPtr, std::pair<SSkelPtr, cv::Point2d> >& getSkelOffsets() const
	{
		return m_skel_offsets;
	}

	void setSkelOffsets(
			const std::map<SSkelPtr, std::pair<SSkelPtr, cv::Point2d> >& skelOffsets)
	{
		m_skel_offsets = skelOffsets;
	}

	const cv::Point3d& getBrf() const
	{
		return m_brf;
	}

	void setBrf(
			const cv::Point3d& brf)
	{
		m_brf = brf;
	}

	const cv::Point3d& getTlb() const
	{
		return m_tlb;
	}

	void setTlb(
			const cv::Point3d& tlb)
	{
		m_tlb = tlb;
	}
};

} /* namespace nl_uu_science_gmt */

#endif /* FRAME_H_ */
