/*
 * types.h
 *
 *  Created on: March, 2016
 *      Author: anonymous ECCV submission 1747
 */

#ifndef TYPES_H_
#define TYPES_H_

#include <opencv2/opencv.hpp>
#include <map>
#include <memory>
#include <string>
#include <utility>
#include <vector>

#ifndef UINT
#define UINT unsigned int
#endif

namespace nl_uu_science_gmt
{
	typedef std::shared_ptr<cv::Mat> MatPtr;

	class Video;
	typedef std::shared_ptr<Video> SVideoPtr;
	typedef std::vector<SVideoPtr> Videos;

	class Frame;
	typedef std::shared_ptr<Frame> SFramePtr;
	typedef std::vector<SFramePtr> Frames;

	class Skeleton;
	typedef std::shared_ptr<Skeleton> SSkelPtr;
	typedef std::vector<SSkelPtr> Skeletons;

	typedef enum _ShakeFiveVersion
	{
		ShakeFiveVersion_Kinect1 = 1,
		ShakeFiveVersion_Kinect2 = 2,
		ShakeFiveVersion_NA = 0
	} ShakeFiveVersion;

	typedef enum _NUI_SKELETON_POSITION_INDEX
	{
		SKELETON_POSITION_HIP_CENTER = 0,
		SKELETON_POSITION_SPINE = (SKELETON_POSITION_HIP_CENTER + 1),
		SKELETON_POSITION_SHOULDER_CENTER = (SKELETON_POSITION_SPINE + 1),
		SKELETON_POSITION_HEAD = (SKELETON_POSITION_SHOULDER_CENTER + 1),
		SKELETON_POSITION_SHOULDER_LEFT = (SKELETON_POSITION_HEAD + 1),
		SKELETON_POSITION_ELBOW_LEFT = (SKELETON_POSITION_SHOULDER_LEFT + 1),
		SKELETON_POSITION_WRIST_LEFT = (SKELETON_POSITION_ELBOW_LEFT + 1),
		SKELETON_POSITION_HAND_LEFT = (SKELETON_POSITION_WRIST_LEFT + 1),
		SKELETON_POSITION_SHOULDER_RIGHT = (SKELETON_POSITION_HAND_LEFT + 1),
		SKELETON_POSITION_ELBOW_RIGHT = (SKELETON_POSITION_SHOULDER_RIGHT + 1),
		SKELETON_POSITION_WRIST_RIGHT = (SKELETON_POSITION_ELBOW_RIGHT + 1),
		SKELETON_POSITION_HAND_RIGHT = (SKELETON_POSITION_WRIST_RIGHT + 1),
		SKELETON_POSITION_HIP_LEFT = (SKELETON_POSITION_HAND_RIGHT + 1),
		SKELETON_POSITION_KNEE_LEFT = (SKELETON_POSITION_HIP_LEFT + 1),
		SKELETON_POSITION_ANKLE_LEFT = (SKELETON_POSITION_KNEE_LEFT + 1),
		SKELETON_POSITION_FOOT_LEFT = (SKELETON_POSITION_ANKLE_LEFT + 1),
		SKELETON_POSITION_HIP_RIGHT = (SKELETON_POSITION_FOOT_LEFT + 1),
		SKELETON_POSITION_KNEE_RIGHT = (SKELETON_POSITION_HIP_RIGHT + 1),
		SKELETON_POSITION_ANKLE_RIGHT = (SKELETON_POSITION_KNEE_RIGHT + 1),
		SKELETON_POSITION_FOOT_RIGHT = (SKELETON_POSITION_ANKLE_RIGHT + 1),

		SKELETON_POSITION_COUNT = (SKELETON_POSITION_FOOT_RIGHT + 1)
	} NUI_SKELETON_POSITION_INDEX;

	typedef std::map<NUI_SKELETON_POSITION_INDEX, std::string> JointNameMap;

	typedef enum _Kinect2JointType
	{
		JointType_K2_SpineBase	= 0,
		JointType_K2_SpineMid	= 1,
		JointType_K2_Neck	= 2,
		JointType_K2_Head	= 3,
		JointType_K2_ShoulderLeft	= 4,
		JointType_K2_ElbowLeft	= 5,
		JointType_K2_WristLeft	= 6,
		JointType_K2_HandLeft	= 7,
		JointType_K2_ShoulderRight	= 8,
		JointType_K2_ElbowRight	= 9,
		JointType_K2_WristRight	= 10,
		JointType_K2_HandRight	= 11,
		JointType_K2_HipLeft	= 12,
		JointType_K2_KneeLeft	= 13,
		JointType_K2_AnkleLeft	= 14,
		JointType_K2_FootLeft	= 15,
		JointType_K2_HipRight	= 16,
		JointType_K2_KneeRight	= 17,
		JointType_K2_AnkleRight	= 18,
		JointType_K2_FootRight	= 19,
		JointType_K2_SpineShoulder	= 20,
		JointType_K2_HandTipLeft	= 21,
		JointType_K2_ThumbLeft	= 22,
		JointType_K2_HandTipRight	= 23,
		JointType_K2_ThumbRight	= 24,
		JointType_K2_Count	= ( JointType_K2_ThumbRight + 1 ) 
	} Kinect2JointType;

	typedef std::map<Kinect2JointType, std::string> Kinect2JointNameMap;

	typedef enum _ACTION_CLASSES_INDEX
	{
		AC_NA,
		AC_STANDING = 1,
		AC_APPROACHING = 2,
		AC_HANDSHAKING = 3,
		AC_HIGHFIVING = 4,
		AC_LEAVING = 5,
		AC_OBJECT_PASSING = 7,
		AC_HUGGING = 8,
		AC_FIST_BUMPING = 9, /* Box */
		AC_KISSING = 10,
		AC_RPS = 11, /* Rock, paper, scissors */
		AC_TU = 12, /* Thumbs Up */
		AC_EXPL = 13, /* Explain a route */
		AC_END = 14,
		AC_KICKING = 15,
		AC_POINTING = 16,
		AC_PUNCHING = 17,
		AC_PUSHING = 18,
		AC_HANDWAVING = 19,
		AC_BOWING = 20,
		AC_BOXING = 21,
		AC_PATTING = 22,
		AC_COUNT = (AC_END + 1)
	} ACTION_CLASSES_INDEX;

	typedef std::map<ACTION_CLASSES_INDEX, std::string> ActionNameMap;

	typedef std::vector<std::pair<NUI_SKELETON_POSITION_INDEX, Kinect2JointType>> Kinect12Map;

	struct FrameGroundTruth
	{
		int fid;
		std::vector<std::pair<int, ACTION_CLASSES_INDEX>> people;
	};
	typedef std::shared_ptr<FrameGroundTruth> FrameGroundTruthPtr;
	typedef std::map<int, FrameGroundTruthPtr> FrameGroundTruths;

	const static cv::Scalar Color_WHITE = CV_RGB(255, 255, 255);
	const static cv::Scalar Color_RED = CV_RGB(255, 0, 0);
	const static cv::Scalar Color_GREEN = CV_RGB(0, 200, 0);
	const static cv::Scalar Color_BLUE = CV_RGB(0, 0, 255);
	const static cv::Scalar Color_YELLOW = CV_RGB(255, 255, 0);
	const static cv::Scalar Color_MAGENTA = CV_RGB(255, 0, 255);
	const static cv::Scalar Color_CYAN = CV_RGB(0, 255, 255);
	const static cv::Scalar Color_OLIVE = CV_RGB(128, 128, 0);
	const static cv::Scalar Color_PURPLE = CV_RGB(128, 0, 128);
	const static cv::Scalar Color_TEAL = CV_RGB(0, 128, 128);
	const static cv::Scalar Color_ORANGE = CV_RGB(255, 165, 0);
	const static cv::Scalar Color_PINK = CV_RGB(255, 192, 203);
	const static cv::Scalar Color_GRAY = CV_RGB(128, 128, 128);
	const static cv::Scalar Color_COBALT = CV_RGB(61, 89, 171);
	const static cv::Scalar Color_KHAKI = CV_RGB(240, 230, 140);
	const static cv::Scalar Color_BROWN = CV_RGB(139, 69, 19);
	const static cv::Scalar Color_BLACK = CV_RGB(0, 0, 0);
}

#endif /* TYPES_H_ */
